import * as THREE from "three";
import { useEffect, useMemo, useRef, useState } from "react";
import { useFrame, useThree } from "@react-three/fiber";
import { useFloorItems, useSelectedAssets, useSimulationPaths, useSocketStore, useToggleView } from "../../../../store/store";
// import { setFloorItemApi } from '../../../../services/factoryBuilder/assest/floorAsset/setFloorItemApi';
import { toast } from "react-toastify";
import * as Types from "../../../../types/world/worldTypes";

function MoveControls({ movedObjects, setMovedObjects, itemsGroupRef, copiedObjects, setCopiedObjects, pastedObjects, setpastedObjects, duplicatedObjects, setDuplicatedObjects, selectionGroup, rotatedObjects, setRotatedObjects, boundingBoxRef }: any) {
    const { camera, controls, gl, scene, pointer, raycaster } = useThree();
    const plane = useMemo(() => new THREE.Plane(new THREE.Vector3(0, 1, 0), 0), []);

    const { toggleView } = useToggleView();
    const { selectedAssets, setSelectedAssets } = useSelectedAssets();
    const { simulationPaths, setSimulationPaths } = useSimulationPaths();
    const { floorItems, setFloorItems } = useFloorItems();
    const { socket } = useSocketStore();
    const itemsData = useRef<Types.FloorItems>([]);

    useEffect(() => {
        if (!camera || !scene || toggleView || !itemsGroupRef.current) return;

        const canvasElement = gl.domElement;
        canvasElement.tabIndex = 0;

        let isMoving = false;

        const onPointerDown = () => {
            isMoving = false;
        };

        const onPointerMove = () => {
            isMoving = true;
        };

        const onPointerUp = (event: PointerEvent) => {
            if (!isMoving && movedObjects.length > 0 && event.button === 0) {
                event.preventDefault();
                placeMovedAssets();
            }
            if (!isMoving && movedObjects.length > 0 && event.button === 2) {
                event.preventDefault();

                clearSelection();
                movedObjects.forEach((asset: any) => {
                    if (itemsGroupRef.current) {
                        itemsGroupRef.current.attach(asset);
                    }
                });

                setFloorItems([...floorItems, ...itemsData.current]);

                setMovedObjects([]);
                itemsData.current = [];
            }
        };

        const onKeyDown = (event: KeyboardEvent) => {
            if (pastedObjects.length > 0 || duplicatedObjects.length > 0 || rotatedObjects.length > 0) return;
            if (event.key.toLowerCase() === "g") {
                if (selectedAssets.length > 0) {
                    moveAssets();
                    itemsData.current = floorItems.filter((item: { modeluuid: string }) => selectedAssets.some((asset: any) => asset.uuid === item.modeluuid));
                }
            }
            if (event.key.toLowerCase() === "escape") {
                event.preventDefault();

                clearSelection();
                movedObjects.forEach((asset: any) => {
                    if (itemsGroupRef.current) {
                        itemsGroupRef.current.attach(asset);
                    }
                });

                setFloorItems([...floorItems, ...itemsData.current]);

                setMovedObjects([]);
                itemsData.current = [];
            }
        };

        if (!toggleView) {
            canvasElement.addEventListener("pointerdown", onPointerDown);
            canvasElement.addEventListener("pointermove", onPointerMove);
            canvasElement.addEventListener("pointerup", onPointerUp);
            canvasElement.addEventListener("keydown", onKeyDown);
        }

        return () => {
            canvasElement.removeEventListener("pointerdown", onPointerDown);
            canvasElement.removeEventListener("pointermove", onPointerMove);
            canvasElement.removeEventListener("pointerup", onPointerUp);
            canvasElement.removeEventListener("keydown", onKeyDown);
        };
    }, [camera, controls, scene, toggleView, selectedAssets, socket, floorItems, pastedObjects, duplicatedObjects, movedObjects, rotatedObjects]);

    const gridSize = 0.25;
    const moveSpeed = 0.25;
    const isGridSnap = false;

    useFrame(() => {
        if (movedObjects.length > 0) {
            const intersectionPoint = new THREE.Vector3();
            raycaster.setFromCamera(pointer, camera);
            const point = raycaster.ray.intersectPlane(plane, intersectionPoint);

            if (point) {
                let targetX = point.x;
                let targetZ = point.z;

                if (isGridSnap) {
                    targetX = Math.round(point.x / gridSize) * gridSize;
                    targetZ = Math.round(point.z / gridSize) * gridSize;
                }

                const position = new THREE.Vector3();
                if (boundingBoxRef.current) {
                    boundingBoxRef.current.getWorldPosition(position);
                    selectionGroup.current.position.lerp(
                        new THREE.Vector3(
                            targetX - (position.x - selectionGroup.current.position.x),
                            selectionGroup.current.position.y,
                            targetZ - (position.z - selectionGroup.current.position.z)
                        ),
                        moveSpeed
                    );
                } else {
                    const box = new THREE.Box3();
                    movedObjects.forEach((obj: THREE.Object3D) => box.expandByObject(obj));
                    const center = new THREE.Vector3();
                    box.getCenter(center);

                    selectionGroup.current.position.lerp(
                        new THREE.Vector3(
                            targetX - (center.x - selectionGroup.current.position.x),
                            selectionGroup.current.position.y,
                            targetZ - (center.z - selectionGroup.current.position.z)
                        ),
                        moveSpeed
                    );
                }
            }
        }
    });


    const moveAssets = () => {
        const updatedItems = floorItems.filter((item: { modeluuid: string }) => !selectedAssets.some((asset: any) => asset.uuid === item.modeluuid));
        setFloorItems(updatedItems);
        setMovedObjects(selectedAssets);
        selectedAssets.forEach((asset: any) => { selectionGroup.current.attach(asset); });
    }

    const placeMovedAssets = () => {
        if (movedObjects.length === 0) return;

        movedObjects.forEach(async (obj: THREE.Object3D) => {
            const worldPosition = new THREE.Vector3();
            obj.getWorldPosition(worldPosition);

            selectionGroup.current.remove(obj);
            obj.position.copy(worldPosition);

            if (itemsGroupRef.current) {

                const newFloorItem: Types.FloorItemType = {
                    modeluuid: obj.uuid,
                    modelname: obj.userData.name,
                    modelfileID: obj.userData.modelId,
                    position: [worldPosition.x, worldPosition.y, worldPosition.z],
                    rotation: { x: obj.rotation.x, y: obj.rotation.y, z: obj.rotation.z, },
                    isLocked: false,
                    isVisible: true
                };

                setFloorItems((prevItems: Types.FloorItems) => {
                    const updatedItems = [...(prevItems || []), newFloorItem];
                    localStorage.setItem("FloorItems", JSON.stringify(updatedItems));
                    return updatedItems;
                });

                let eventData: Types.ConveyorEventsSchema | Types.VehicleEventsSchema | undefined = simulationPaths.find((events) => events.modeluuid === obj.userData.modeluuid);

                const email = localStorage.getItem("email");
                const organization = email ? email.split("@")[1].split(".")[0] : "default";

                if (eventData) {
                    if (eventData.type === 'Conveyor' && eventData) {

                        const backendEventData = {
                            type: 'Conveyor',
                            points: eventData.points,
                            speed: (eventData as Types.ConveyorEventsSchema)?.speed
                        };

                        //REST

                        // await setFloorItemApi(
                        //     organization,
                        //     obj.uuid,
                        //     obj.userData.name,
                        //     [worldPosition.x, worldPosition.y, worldPosition.z],
                        //     { "x": obj.rotation.x, "y": obj.rotation.y, "z": obj.rotation.z },
                        //     obj.userData.modelId,
                        //     false,
                        //     true,
                        //     backendEventData
                        // );

                        //SOCKET

                        const data = {
                            organization,
                            modeluuid: newFloorItem.modeluuid,
                            modelname: newFloorItem.modelname,
                            modelfileID: newFloorItem.modelfileID,
                            position: newFloorItem.position,
                            rotation: { x: obj.rotation.x, y: obj.rotation.y, z: obj.rotation.z },
                            isLocked: false,
                            isVisible: true,
                            eventData: backendEventData,
                            socketId: socket.id,
                        };

                        const newEventData: any = backendEventData;
                        newEventData.modeluuid = newFloorItem.modeluuid;
                        newEventData.modelName = newFloorItem.modelname;
                        newEventData.position = newFloorItem.position;
                        newEventData.rotation = [obj.rotation.x, obj.rotation.y, obj.rotation.z];

                        setSimulationPaths((prevEvents: (Types.ConveyorEventsSchema | Types.VehicleEventsSchema)[]) => {
                            const updatedEvents = (prevEvents || []).map(event =>
                                event.modeluuid === newFloorItem.modeluuid
                                    ? { ...event, ...newEventData }
                                    : event
                            );
                            return updatedEvents;
                        });

                        // socket.emit("v2:model-asset:add", data);
                    }
                } else {


                    //REST

                    // await setFloorItemApi(
                    //     organization,
                    //     obj.uuid,
                    //     obj.userData.name,
                    //     [worldPosition.x, worldPosition.y, worldPosition.z],
                    //     { "x": obj.rotation.x, "y": obj.rotation.y, "z": obj.rotation.z },
                    //     obj.userData.modelId,
                    //     false,
                    //     true,
                    // );

                    //SOCKET

                    const data = {
                        organization,
                        modeluuid: newFloorItem.modeluuid,
                        modelname: newFloorItem.modelname,
                        modelfileID: newFloorItem.modelfileID,
                        position: newFloorItem.position,
                        rotation: { x: obj.rotation.x, y: obj.rotation.y, z: obj.rotation.z },
                        isLocked: false,
                        isVisible: true,
                        socketId: socket.id,
                    };

                    socket.emit("v2:model-asset:add", data);

                }

                itemsGroupRef.current.add(obj);
            }
        });
        toast.success("Object moved!");

        itemsData.current = [];
        clearSelection();
    }

    const clearSelection = () => {
        selectionGroup.current.children = [];
        selectionGroup.current.position.set(0, 0, 0);
        selectionGroup.current.rotation.set(0, 0, 0);
        setpastedObjects([]);
        setDuplicatedObjects([]);
        setMovedObjects([]);
        setRotatedObjects([]);
        setSelectedAssets([]);
    }

    return (
        <></>
    )
}

export default MoveControls